@@ -6,7 +6,7 @@
 
 package Graph::Easy::Node;
 
-$VERSION = '0.34';
+$VERSION = '0.35';
 
 use Graph::Easy::Base;
 use Graph::Easy::Attributes;
@@ -24,7 +24,6 @@ sub _init
   
   $self->{name} = 'Node #' . $self->{id};
   
-  # attributes
   $self->{att} = { };
   $self->{class} = 'node';		# default class
 
@@ -38,16 +37,17 @@ sub _init
     $self->{$k} = $args->{$k} if $k eq 'name';
     $self->{att}->{$k} = $args->{$k} if $k eq 'label';
     }
-  
-  $self->{x} = 0;
-  $self->{y} = 0;
- 
+
   # These are undef (to save memory) until needed: 
-  # $self->{children} = {};
-  # $self->{dx} = 0;		# relative to no other node
-  # $self->{dy} = 0;
-  # $self->{origin} = 0;
-  # $self->{group} = undef;
+  #  $self->{children} = {};
+  #  $self->{dx} = 0;		# relative to no other node
+  #  $self->{dy} = 0;
+  #  $self->{origin} = 0;
+  #  $self->{group} = undef;
+  #  $self->{parent} = $graph or $group;
+  # Mark as not yet laid out: 
+  #  $self->{x} = 0;
+  #  $self->{y} = 0;
   
   $self;
   }
@@ -1545,7 +1545,7 @@ sub pos
   {
   my $self = shift;
 
-  ($self->{x}, $self->{y});
+  ($self->{x} || 0, $self->{y} || 0);
   }
 
 sub offset
@@ -1796,7 +1796,7 @@ sub edges
   # no graph, no dice
   return unless ref $self->{graph};
 
-  values %{$self->{edges}};
+  wantarray ? values %{$self->{edges}} : scalar keys %{$self->{edges}};
   }
 
 sub sorted_successors
@@ -1858,6 +1858,37 @@ sub has_predecessors
   0;						# found none
   }
 
+sub has_as_predecessor
+  {
+  # return true if other is a predecessor of node
+  my ($self,$other) = @_;
+
+  return () unless defined $self->{graph};
+
+  for my $edge (values %{$self->{edges}})
+    {
+    return 1 if 
+	$edge->{to} == $self && $edge->{from} == $other;	# found one
+    }
+  0;						# found none
+  }
+
+sub has_as_successor
+  {
+  # return true if other is a successor of node
+  my ($self,$other) = @_;
+
+  return () unless defined $self->{graph};
+
+  for my $edge (values %{$self->{edges}})
+    {
+    return 1 if
+	$edge->{from} == $self && $edge->{to} == $other;	# found one
+
+    }
+  0;						# found none
+  }
+
 #############################################################################
 # relatively placed nodes
 
@@ -2257,15 +2288,30 @@ of the specific class the node is in, then this will be returned.
 
 =head2 get_attributes()
 
-	my $attr = $node->get_attributes();
+        my $att = $object->get_attributes();
 
-Return all the set attributes of the node.
+Return all effective attributes on this object (graph/node/group/edge) as
+an anonymous hash ref. This respects inheritance and default values.
+
+See also L<raw_attributes()>.
+
+=head2 raw_attributes()
+
+        my $att = $object->get_attributes();
+
+Return all set attributes on this object (graph/node/group/edge) as
+an anonymous hash ref. This respects inheritance, but does not include
+default values for unset attributes.
+
+See also L<get_attributes()>.
 
 =head2 attributes_as_txt
 
 	my $txt = $node->attributes_as_txt();
 
-Return the attributes of this node as text description.
+Return the attributes of this node as text description. This is used
+by the C<< $graph->as_txt() >> code and there should be no reason
+to use this function on your own.
 
 =head2 set_attribute()
 
@@ -2330,7 +2376,9 @@ happening.
 
 	my $name = $node->name();
 
-Return the name of the node.
+Return the name of the node. In a graph, each node has a unique name,
+which, unless a node label is set, will be displayed when rendering the
+graph.
 
 =head2 label()
 
@@ -2368,8 +2416,7 @@ If no title was set (or autogenerated), will return an empty string.
 
 If the optional parameter is true, then the title will returned 'raw',
 that is any potential escape of the form C<\N>, C<\E>, C<\G>, C<\T>
-or C<\H> will not be left alone and not be replaced.
-
+or C<\H> will be left alone and not be replaced.
 
 =head2 link()
 
@@ -2384,7 +2431,6 @@ If the optional parameter is true, then the link will returned 'raw',
 that is any potential escape of the form C<\N>, C<\E>, C<\G>, C<\T>
 or C<\H> will not be left alone and not be replaced.
 
-
 =head2 dimensions()
 
 	my ($w,$h) = $node->dimensions();
@@ -2453,7 +2499,7 @@ objects, and true for C<Graph::Easy::Node::Anon>.
 	my ($x,$y) = $node->pos();
 
 Returns the position of the node. Initially, this is undef, and will be
-set from L<Graph::Easy::layout()>.
+set from L<Graph::Easy::layout()>. Only valid during the layout phase.
 
 =head2 offset()
 
@@ -2467,20 +2513,20 @@ the origin node was not sset.
 	my $x = $node->x();
 
 Returns the X position of the node. Initially, this is undef, and will be
-set from L<Graph::Easy::layout()>.
+set from L<Graph::Easy::layout()>. Only valid during the layout phase.
 
 =head2 y()
 
 	my $y = $node->y();
 
 Returns the Y position of the node. Initially, this is undef, and will be
-set from L<Graph::Easy::layout()>.
+set from L<Graph::Easy::layout()>. Only valid during the layout phase.
 
 =head2 id()
 
 	my $id = $node->id();
 
-Returns the node's unique ID number.
+Returns the node's unique, internal ID number.
 
 =head2 connections()
 
@@ -2534,11 +2580,32 @@ Returns all nodes (as objects) that we are linking to.
 Return successors of the node sorted by their chain value
 (e.g. successors with more successors first). 
 
+=head2 has_as_successor()
+
+	if ($node->has_as_successor($other))
+	  {
+	  ...
+	  }
+
+Returns true if C<$other> ( a node or group) is a successor of node, that is if
+there is an edge leading from node to C<$other>.
+
+=head2 has_as_predecessor()
+
+	if ($node->has_as_predecessor($other))
+	  {
+	  ...
+	  }
+
+Returns true if the node has C<$other> (a group or node) as predecessor, that
+is if there is an edge leading from C<$other> to node.
+
 =head2 edges_to()
 
 	my @edges = $node->edges_to($other_node);
 
-Returns all the edge objects that start at C<< $node >> and go to C<< $other_node >>.
+Returns all the edges (as objects) that start at C<< $node >> and go to
+C<< $other_node >>.
 
 =head2 shared_edges()
 
